==> qiskit-nature-0.5.1.tar.gz: qiskit-nature-0.5.1/test/second_q/hamiltonians/test_quadratic_hamiltonian.py <==
# This code is part of Qiskit.
#
# (C) Copyright IBM 2021, 2022.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.

"""Tests for QuadraticHamiltonian"""

from test import QiskitNatureTestCase
from test.nature_random import random_antisymmetric_matrix

import numpy as np
from ddt import data, ddt
from qiskit.quantum_info import random_hermitian

from qiskit_nature.second_q.mappers import QubitConverter
from qiskit_nature.second_q.mappers import JordanWignerMapper
from qiskit_nature.second_q.hamiltonians import QuadraticHamiltonian
from qiskit_nature.second_q.operators import FermionicOp


@ddt
class TestQuadraticHamiltonian(QiskitNatureTestCase):
    """QuadraticHamiltonian tests."""

    def test_init(self):
        """Test initialization."""
        hermitian_part = np.eye(2)
        antisymmetric_part = np.array([[0, 1], [-1, 0]])
        constant = 1.0
        zero = np.zeros((2, 2))

        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part, constant)
        np.testing.assert_allclose(quad_ham.hermitian_part, hermitian_part)
        np.testing.assert_allclose(quad_ham.antisymmetric_part, antisymmetric_part)
        np.testing.assert_allclose(quad_ham.constant, constant)
        self.assertEqual(quad_ham.num_modes, 2)

        quad_ham = QuadraticHamiltonian(None, antisymmetric_part)
        np.testing.assert_allclose(quad_ham.hermitian_part, zero)
        self.assertEqual(quad_ham.num_modes, 2)

        quad_ham = QuadraticHamiltonian(hermitian_part)
        np.testing.assert_allclose(quad_ham.antisymmetric_part, zero)
        self.assertEqual(quad_ham.num_modes, 2)

        quad_ham = QuadraticHamiltonian(num_modes=2)
        np.testing.assert_allclose(quad_ham.hermitian_part, zero)
        np.testing.assert_allclose(quad_ham.antisymmetric_part, zero)
        self.assertEqual(quad_ham.num_modes, 2)

        with self.assertRaisesRegex(ValueError, "specified"):
            _ = QuadraticHamiltonian()

    def test_conserves_particle_number(self):
        """Test particle number conservation predicate."""
        hermitian_part = np.eye(2)
        antisymmetric_part = np.array([[0, 1], [-1, 0]])

        quad_ham = QuadraticHamiltonian(hermitian_part)
        assert quad_ham.conserves_particle_number()

        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part)
        assert not quad_ham.conserves_particle_number()

    def test_diagonalizing_bogoliubov_transform(self):
        """Test diagonalizing Bogoliubov transform."""
        hermitian_part = np.array(
            [[0.0, 1.0, 0.0], [1.0, 0.0, 1.0], [0.0, 1.0, 0.0]], dtype=complex
        )
        antisymmetric_part = np.array(
            [[0.0, 1.0j, 0.0], [-1.0j, 0.0, 1.0j], [0.0, -1.0j, 0.0]], dtype=complex
        )
        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part)
        (
            transformation_matrix,
            orbital_energies,
            transformed_constant,
        ) = quad_ham.diagonalizing_bogoliubov_transform()

        # test that the transformation diagonalizes the Hamiltonian
        left = transformation_matrix[:, :3]
        right = transformation_matrix[:, 3:]
        full_transformation_matrix = np.block([[left, right], [right.conj(), left.conj()]])
        eye = np.eye(3, dtype=complex)
        majorana_basis = np.block([[eye, eye], [1j * eye, -1j * eye]]) / np.sqrt(2)
        basis_change = majorana_basis @ full_transformation_matrix @ majorana_basis.T.conj()
        majorana_matrix, majorana_constant = quad_ham.majorana_form()
        canonical = basis_change @ majorana_matrix @ basis_change.T

        zero = np.zeros((3, 3))
        diagonal = np.diag(orbital_energies)
        expected = np.block([[zero, diagonal], [-diagonal, zero]])

        np.testing.assert_allclose(orbital_energies, np.sort(orbital_energies))
        np.testing.assert_allclose(canonical, expected, atol=1e-7)
        np.testing.assert_allclose(
            transformed_constant, majorana_constant - 0.5 * np.sum(orbital_energies)
        )

        # confirm eigenvalues match with Jordan-Wigner transformed Hamiltonian
        hamiltonian_jw = (
            QubitConverter(mapper=JordanWignerMapper())
            .convert(quad_ham.second_q_op())
            .primitive.to_matrix()
        )
        eigs, _ = np.linalg.eigh(hamiltonian_jw)
        expected_eigs = np.array(
            [
                np.sum(orbital_energies[list(occupied_orbitals)]) + transformed_constant
                for occupied_orbitals in [(), (0,), (1,), (2,), (0, 1), (0, 2), (1, 2), (0, 1, 2)]
            ]
        )
        np.testing.assert_allclose(np.sort(eigs), np.sort(expected_eigs), atol=1e-7)

    @data(1, 2, 3, 4, 5, 6)
    def test_diagonalizing_bogoliubov_transform_particle_number_conserving(self, n_orbitals):
        """Test diagonalizing Bogoliubov transform, particle-number-conserving case."""
        hermitian_part = random_hermitian(n_orbitals).data
        constant = np.random.uniform(-10, 10)
        quad_ham = QuadraticHamiltonian(hermitian_part, constant=constant)
        (
            transformation_matrix,
            orbital_energies,
            transformed_constant,
        ) = quad_ham.diagonalizing_bogoliubov_transform()
        diagonalized = transformation_matrix @ hermitian_part.T @ transformation_matrix.T.conj()

        np.testing.assert_allclose(orbital_energies, np.sort(orbital_energies))
        np.testing.assert_allclose(diagonalized, np.diag(orbital_energies), atol=1e-7)
        np.testing.assert_allclose(transformed_constant, constant)

    @data(2, 3, 4, 5, 6)
    def test_diagonalizing_bogoliubov_transform_non_particle_number_conserving(self, n_orbitals):
        """Test diagonalizing Bogoliubov transform, non-particle-number-conserving case."""
        hermitian_part = random_hermitian(n_orbitals).data
        antisymmetric_part = random_antisymmetric_matrix(n_orbitals, seed=9985)
        constant = np.random.uniform(-10, 10)
        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part, constant=constant)
        (
            transformation_matrix,
            orbital_energies,
            transformed_constant,
        ) = quad_ham.diagonalizing_bogoliubov_transform()

        left = transformation_matrix[:, :n_orbitals]
        right = transformation_matrix[:, n_orbitals:]
        full_transformation_matrix = np.block([[left, right], [right.conj(), left.conj()]])
        eye = np.eye(n_orbitals, dtype=complex)
        majorana_basis = np.block([[eye, eye], [1j * eye, -1j * eye]]) / np.sqrt(2)
        basis_change = majorana_basis @ full_transformation_matrix @ majorana_basis.T.conj()
        majorana_matrix, majorana_constant = quad_ham.majorana_form()
        canonical = basis_change @ majorana_matrix @ basis_change.T

        zero = np.zeros((n_orbitals, n_orbitals))
        diagonal = np.diag(orbital_energies)
        expected = np.block([[zero, diagonal], [-diagonal, zero]])

        np.testing.assert_allclose(orbital_energies, np.sort(orbital_energies))
        np.testing.assert_allclose(canonical, expected, atol=1e-7)
        np.testing.assert_allclose(
            transformed_constant, majorana_constant - 0.5 * np.sum(orbital_energies)
        )

    def test_fermionic_op(self):
        """Test conversion to FermionicOp."""
        hermitian_part = np.array([[1, 2j], [-2j, 3]])
        antisymmetric_part = np.array([[0, 4j], [-4j, 0]])
        constant = 5.0
        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part, constant)
        fermionic_op = quad_ham.second_q_op()
        expected_terms = {
            "": 5.0,
            "+_0 -_0": 1.0,
            "+_1 -_1": 3.0,
            "+_0 -_1": 2j,
            "-_0 +_1": 2j,
            "+_0 +_1": 4j,
            "-_0 -_1": 4j,
        }
        expected_op = FermionicOp(expected_terms, num_spin_orbitals=2)
        matrix = fermionic_op.to_matrix(sparse=False)
        expected_matrix = expected_op.to_matrix(sparse=False)
        np.testing.assert_allclose(matrix, expected_matrix)

    def test_validate(self):
        """Test input validation."""
        mat = np.array([[1, 2], [3, 4]])
        _ = QuadraticHamiltonian(hermitian_part=mat, antisymmetric_part=None, validate=False)
        with self.assertRaisesRegex(ValueError, "Hermitian"):
            _ = QuadraticHamiltonian(hermitian_part=mat, antisymmetric_part=None)
        with self.assertRaisesRegex(ValueError, "Antisymmetric"):
            _ = QuadraticHamiltonian(hermitian_part=None, antisymmetric_part=mat)

        hermitian_part = np.array([[1, 2j], [-2j, 3]])
        antisymmetric_part = np.array([[0, 4, 0], [-4, 0, 4], [0, -4, 0]])
        with self.assertRaisesRegex(ValueError, "same shape"):
            _ = QuadraticHamiltonian(hermitian_part, antisymmetric_part)
        with self.assertRaisesRegex(ValueError, "num_modes"):
            _ = QuadraticHamiltonian(hermitian_part, num_modes=5)
        with self.assertRaisesRegex(ValueError, "num_modes"):
            _ = QuadraticHamiltonian(antisymmetric_part=antisymmetric_part, num_modes=5)

    def test_addition(self):
        """Test addition."""
        quad_ham_1 = QuadraticHamiltonian(
            np.array([[1, 2j], [-2j, 3]]), np.array([[0, 4], [-4, 0]]), 5.0
        )
        quad_ham_2 = QuadraticHamiltonian(
            np.array([[6, 7j], [-7j, 8]]), np.array([[0, 9j], [-9j, 0]]), 10.0
        )

        quad_ham_sum = quad_ham_1 + quad_ham_2
        expected = QuadraticHamiltonian(
            np.array([[7, 9j], [-9j, 11]]), np.array([[0, 4 + 9j], [-4 - 9j, 0]]), 15.0
        )
        self.assertEqual(quad_ham_sum, expected)

        quad_ham_difference = quad_ham_1 - quad_ham_2
        expected = QuadraticHamiltonian(
            np.array([[-5, -5j], [5j, -5]]), np.array([[0, 4 - 9j], [-4 + 9j, 0]]), -5.0
        )
        self.assertEqual(quad_ham_difference, expected)

    def test_multiplication(self):
        """Test multiplication by a scalar."""
        quad_ham = QuadraticHamiltonian(
            np.array([[1, 2j], [-2j, 3]]), np.array([[0, 4], [-4, 0]]), 5.0
        )

        quad_ham_scaled = 2 * quad_ham
        expected = QuadraticHamiltonian(
            np.array([[2, 4j], [-4j, 6]]), np.array([[0, 8], [-8, 0]]), 10.0
        )
        self.assertEqual(quad_ham_scaled, expected)

        quad_ham_scaled = quad_ham * 2
        expected = QuadraticHamiltonian(
            np.array([[2, 4j], [-4j, 6]]), np.array([[0, 8], [-8, 0]]), 10.0
        )
        self.assertEqual(quad_ham_scaled, expected)

    def test_repr(self):
        """Test repr"""
        quad_ham = QuadraticHamiltonian(
            np.array([[1, 2j], [-2j, 3]]), np.array([[0, 4.0], [-4.0, 0]]), 5.0
        )
        self.assertEqual(
            repr(quad_ham),
            "QuadraticHamiltonian("
            "hermitian_part=array([[ 1.+0.j,  0.+2.j],\n       [-0.-2.j,  3.+0.j]]), "
            "antisymmetric_part=array([[ 0.,  4.],\n       [-4.,  0.]]), "
            "constant=5.0, num_modes=2)",
        )

==> qiskit-nature-0.6.1.tar.gz: qiskit-nature-0.6.1/test/second_q/hamiltonians/test_quadratic_hamiltonian.py <==
# This code is part of Qiskit.
#
# (C) Copyright IBM 2021, 2023.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.

"""Tests for QuadraticHamiltonian"""

from test import QiskitNatureTestCase

import numpy as np
from ddt import data, ddt
from qiskit.quantum_info import random_hermitian
from qiskit.opflow import PauliSumOp

from qiskit_nature.second_q.hamiltonians import QuadraticHamiltonian
from qiskit_nature.second_q.mappers import JordanWignerMapper, QubitConverter
from qiskit_nature.second_q.operators import FermionicOp
from qiskit_nature.settings import settings
from qiskit_nature.testing import random_antisymmetric_matrix


@ddt
class TestQuadraticHamiltonian(QiskitNatureTestCase):
    """QuadraticHamiltonian tests."""

    def test_init(self):
        """Test initialization."""
        hermitian_part = np.eye(2)
        antisymmetric_part = np.array([[0, 1], [-1, 0]])
        constant = 1.0
        zero = np.zeros((2, 2))

        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part, constant)
        np.testing.assert_allclose(quad_ham.hermitian_part, hermitian_part)
        np.testing.assert_allclose(quad_ham.antisymmetric_part, antisymmetric_part)
        np.testing.assert_allclose(quad_ham.constant, constant)
        self.assertEqual(quad_ham.num_modes, 2)

        quad_ham = QuadraticHamiltonian(None, antisymmetric_part)
        np.testing.assert_allclose(quad_ham.hermitian_part, zero)
        self.assertEqual(quad_ham.num_modes, 2)

        quad_ham = QuadraticHamiltonian(hermitian_part)
        np.testing.assert_allclose(quad_ham.antisymmetric_part, zero)
        self.assertEqual(quad_ham.num_modes, 2)

        quad_ham = QuadraticHamiltonian(num_modes=2)
        np.testing.assert_allclose(quad_ham.hermitian_part, zero)
        np.testing.assert_allclose(quad_ham.antisymmetric_part, zero)
        self.assertEqual(quad_ham.num_modes, 2)

        with self.assertRaisesRegex(ValueError, "specified"):
            _ = QuadraticHamiltonian()

    def test_conserves_particle_number(self):
        """Test particle number conservation predicate."""
        hermitian_part = np.eye(2)
        antisymmetric_part = np.array([[0, 1], [-1, 0]])

        quad_ham = QuadraticHamiltonian(hermitian_part)
        assert quad_ham.conserves_particle_number()

        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part)
        assert not quad_ham.conserves_particle_number()

    def test_diagonalizing_bogoliubov_transform(self):
        """Test diagonalizing Bogoliubov transform."""
        hermitian_part = np.array(
            [[0.0, 1.0, 0.0], [1.0, 0.0, 1.0], [0.0, 1.0, 0.0]], dtype=complex
        )
        antisymmetric_part = np.array(
            [[0.0, 1.0j, 0.0], [-1.0j, 0.0, 1.0j], [0.0, -1.0j, 0.0]], dtype=complex
        )
        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part)
        (
            transformation_matrix,
            orbital_energies,
            transformed_constant,
        ) = quad_ham.diagonalizing_bogoliubov_transform()

        # test that the transformation diagonalizes the Hamiltonian
        left = transformation_matrix[:, :3]
        right = transformation_matrix[:, 3:]
        full_transformation_matrix = np.block([[left, right], [right.conj(), left.conj()]])
        eye = np.eye(3, dtype=complex)
        majorana_basis = np.block([[eye, eye], [1j * eye, -1j * eye]]) / np.sqrt(2)
        basis_change = majorana_basis @ full_transformation_matrix @ majorana_basis.T.conj()
        majorana_matrix, majorana_constant = quad_ham.majorana_form()
        canonical = basis_change @ majorana_matrix @ basis_change.T

        zero = np.zeros((3, 3))
        diagonal = np.diag(orbital_energies)
        expected = np.block([[zero, diagonal], [-diagonal, zero]])

        np.testing.assert_allclose(orbital_energies, np.sort(orbital_energies))
        np.testing.assert_allclose(canonical, expected, atol=1e-7)
        np.testing.assert_allclose(
            transformed_constant, majorana_constant - 0.5 * np.sum(orbital_energies)
        )

        # confirm eigenvalues match with Jordan-Wigner transformed Hamiltonian
        qubit_op = QubitConverter(mapper=JordanWignerMapper()).convert(quad_ham.second_q_op())
        if isinstance(qubit_op, PauliSumOp):
            qubit_op = qubit_op.primitive
        hamiltonian_jw = qubit_op.to_matrix()
        eigs, _ = np.linalg.eigh(hamiltonian_jw)
        expected_eigs = np.array(
            [
                np.sum(orbital_energies[list(occupied_orbitals)]) + transformed_constant
                for occupied_orbitals in [(), (0,), (1,), (2,), (0, 1), (0, 2), (1, 2), (0, 1, 2)]
            ]
        )
        np.testing.assert_allclose(np.sort(eigs), np.sort(expected_eigs), atol=1e-7)

    @data(1, 2, 3, 4, 5, 6)
    def test_diagonalizing_bogoliubov_transform_particle_number_conserving(self, n_orbitals):
        """Test diagonalizing Bogoliubov transform, particle-number-conserving case."""
        hermitian_part = random_hermitian(n_orbitals).data
        constant = np.random.uniform(-10, 10)
        quad_ham = QuadraticHamiltonian(hermitian_part, constant=constant)
        (
            transformation_matrix,
            orbital_energies,
            transformed_constant,
        ) = quad_ham.diagonalizing_bogoliubov_transform()
        diagonalized = transformation_matrix @ hermitian_part.T @ transformation_matrix.T.conj()

        np.testing.assert_allclose(orbital_energies, np.sort(orbital_energies))
        np.testing.assert_allclose(diagonalized, np.diag(orbital_energies), atol=1e-7)
        np.testing.assert_allclose(transformed_constant, constant)

    @data(2, 3, 4, 5, 6)
    def test_diagonalizing_bogoliubov_transform_non_particle_number_conserving(self, n_orbitals):
        """Test diagonalizing Bogoliubov transform, non-particle-number-conserving case."""
        hermitian_part = random_hermitian(n_orbitals).data
        antisymmetric_part = random_antisymmetric_matrix(n_orbitals, seed=9985)
        constant = np.random.uniform(-10, 10)
        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part, constant=constant)
        (
            transformation_matrix,
            orbital_energies,
            transformed_constant,
        ) = quad_ham.diagonalizing_bogoliubov_transform()

        left = transformation_matrix[:, :n_orbitals]
        right = transformation_matrix[:, n_orbitals:]
        full_transformation_matrix = np.block([[left, right], [right.conj(), left.conj()]])
        eye = np.eye(n_orbitals, dtype=complex)
        majorana_basis = np.block([[eye, eye], [1j * eye, -1j * eye]]) / np.sqrt(2)
        basis_change = majorana_basis @ full_transformation_matrix @ majorana_basis.T.conj()
        majorana_matrix, majorana_constant = quad_ham.majorana_form()
        canonical = basis_change @ majorana_matrix @ basis_change.T

        zero = np.zeros((n_orbitals, n_orbitals))
        diagonal = np.diag(orbital_energies)
        expected = np.block([[zero, diagonal], [-diagonal, zero]])

        np.testing.assert_allclose(orbital_energies, np.sort(orbital_energies))
        np.testing.assert_allclose(canonical, expected, atol=1e-7)
        np.testing.assert_allclose(
            transformed_constant, majorana_constant - 0.5 * np.sum(orbital_energies)
        )

    def test_fermionic_op(self):
        """Test conversion to FermionicOp."""
        hermitian_part = np.array([[1, 2j], [-2j, 3]])
        antisymmetric_part = np.array([[0, 4j], [-4j, 0]])
        constant = 5.0
        quad_ham = QuadraticHamiltonian(hermitian_part, antisymmetric_part, constant)
        fermionic_op = quad_ham.second_q_op()
        expected_terms = {
            "": 5.0,
            "+_0 -_0": 1.0,
            "+_0 -_1": 2j,
            "+_1 -_0": -2j,
            "+_1 -_1": 3.0,
            "+_0 +_1": 2j,
            "+_1 +_0": -2j,
            "-_0 -_1": 2j,
            "-_1 -_0": -2j,
        }
        expected_op = FermionicOp(expected_terms, num_spin_orbitals=2)
        self.assertTrue(expected_op.equiv(fermionic_op))

    def test_validate(self):
        """Test input validation."""
        mat = np.array([[1, 2], [3, 4]])
        _ = QuadraticHamiltonian(hermitian_part=mat, antisymmetric_part=None, validate=False)
        with self.assertRaisesRegex(ValueError, "Hermitian"):
            _ = QuadraticHamiltonian(hermitian_part=mat, antisymmetric_part=None)
        with self.assertRaisesRegex(ValueError, "Antisymmetric"):
            _ = QuadraticHamiltonian(hermitian_part=None, antisymmetric_part=mat)

        hermitian_part = np.array([[1, 2j], [-2j, 3]])
        antisymmetric_part = np.array([[0, 4, 0], [-4, 0, 4], [0, -4, 0]])
        with self.assertRaisesRegex(ValueError, "same shape"):
            _ = QuadraticHamiltonian(hermitian_part, antisymmetric_part)
        with self.assertRaisesRegex(ValueError, "num_modes"):
            _ = QuadraticHamiltonian(hermitian_part, num_modes=5)
        with self.assertRaisesRegex(ValueError, "num_modes"):
            _ = QuadraticHamiltonian(antisymmetric_part=antisymmetric_part, num_modes=5)

    def test_addition(self):
        """Test addition."""
        quad_ham_1 = QuadraticHamiltonian(
            np.array([[1, 2j], [-2j, 3]]), np.array([[0, 4], [-4, 0]]), 5.0
        )
        quad_ham_2 = QuadraticHamiltonian(
            np.array([[6, 7j], [-7j, 8]]), np.array([[0, 9j], [-9j, 0]]), 10.0
        )

        quad_ham_sum = quad_ham_1 + quad_ham_2
        expected = QuadraticHamiltonian(
            np.array([[7, 9j], [-9j, 11]]), np.array([[0, 4 + 9j], [-4 - 9j, 0]]), 15.0
        )
        self.assertEqual(quad_ham_sum, expected)

        quad_ham_difference = quad_ham_1 - quad_ham_2
        expected = QuadraticHamiltonian(
            np.array([[-5, -5j], [5j, -5]]), np.array([[0, 4 - 9j], [-4 + 9j, 0]]), -5.0
        )
        self.assertEqual(quad_ham_difference, expected)

    def test_multiplication(self):
        """Test multiplication by a scalar."""
        quad_ham = QuadraticHamiltonian(
            np.array([[1, 2j], [-2j, 3]]), np.array([[0, 4], [-4, 0]]), 5.0
        )

        quad_ham_scaled = 2 * quad_ham
        expected = QuadraticHamiltonian(
            np.array([[2, 4j], [-4j, 6]]), np.array([[0, 8], [-8, 0]]), 10.0
        )
        self.assertEqual(quad_ham_scaled, expected)

        quad_ham_scaled = quad_ham * 2
        expected = QuadraticHamiltonian(
            np.array([[2, 4j], [-4j, 6]]), np.array([[0, 8], [-8, 0]]), 10.0
        )
        self.assertEqual(quad_ham_scaled, expected)

    @data(True, False)
    def test_repr(self, tensor_unwrapping):
        """Test repr"""
        aux = settings.tensor_unwrapping
        try:
            settings.tensor_unwrapping = tensor_unwrapping
            quad_ham = QuadraticHamiltonian(
                np.array([[1, 2j], [-2j, 3]]), np.array([[0, 4.0], [-4.0, 0]]), 5.0
            )

            if not tensor_unwrapping:
                self.assertEqual(
                    repr(quad_ham),
                    "QuadraticHamiltonian("
                    "hermitian_part=array([[ 1.+0.j,  0.+2.j],\n       [-0.-2.j,  3.+0.j]]), "
                    "antisymmetric_part=array([[ 0.,  4.],\n       [-4.,  0.]]), "
                    "constant=array(5.), num_modes=2)",
                )
            else:
                self.assertEqual(
                    repr(quad_ham),
                    "QuadraticHamiltonian("
                    "hermitian_part=array([[ 1.+0.j,  0.+2.j],\n       [-0.-2.j,  3.+0.j]]), "
                    "antisymmetric_part=array([[ 0.,  4.],\n       [-4.,  0.]]), "
                    "constant=5.0, num_modes=2)",
                )
        finally:
            settings.tensor_unwrapping = aux
